/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#ifdef MYFILE_USB_OTG
#include "mf-main.h"
#include "mf-otg.h"
#include "mf-widget.h"
#include "mf-fs-util.h"
#include "mf-fm-svc-wrapper.h"

void mf_otg_list_clean(void *data, mf_otg_list_e list_type)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;
	switch (list_type) {
	case MF_OTG_LIST_DEV:
		EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
			if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
				free(((mf_otg_node *)node)->name);
				((mf_otg_node *)node)->name = NULL;
			}
		}

		if (ap->mf_FileOperation.otg_dev_list) {
			eina_list_free(ap->mf_FileOperation.otg_dev_list);
			ap->mf_FileOperation.otg_dev_list = NULL;
		}
		break;
	case MF_OTG_LIST_REMOVED:
		EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_removed_list, l, node) {
			if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
				free(((mf_otg_node *)node)->name);
				((mf_otg_node *)node)->name = NULL;
			}
		}

		eina_list_free(ap->mf_FileOperation.otg_dev_removed_list);
		ap->mf_FileOperation.otg_dev_removed_list = NULL;
		break;
	default:
		break;
	}
}

int mf_otg_init(void *data)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_FileOperation.otg_dev_list != NULL)
		mf_otg_list_clean(ap, MF_OTG_LIST_DEV);

	if(ap->mf_FileOperation.otg_dev_removed_list != NULL)
		mf_otg_list_clean(ap, MF_OTG_LIST_REMOVED);

	DIR *pDir = NULL;
	struct dirent *ent;
	pDir = opendir(OTG_FOLDER);

	if (pDir == NULL) {
		return	MYFILE_ERR_DIR_OPEN_FAIL;
	}

	while ((ent = readdir(pDir)) != NULL) {
		mf_otg_node *pNode = NULL;

		if (g_strcmp0(ent->d_name, ".") == 0 || g_strcmp0(ent->d_name, "..") == 0) {
			continue;
		}

		if ((ent->d_type & DT_DIR) == 0 && (ent->d_type & DT_REG) == 0) {
			continue;
		}

		pNode = (mf_otg_node *) malloc(sizeof(mf_otg_node));

		if (pNode == NULL) {
			continue;
		}
		memset(pNode, 0, sizeof(mf_otg_node));
		/* set name */
		pNode->name = g_strconcat(OTG_FOLDER, "/", ent->d_name, NULL);
		/* set flag */
		pNode->flag_using = false;
		pNode->flag_editstart = false;

		ap->mf_FileOperation.otg_dev_list = eina_list_append(ap->mf_FileOperation.otg_dev_list, pNode);
	}
	closedir(pDir);
	return MYFILE_ERR_NONE;
}

bool mf_otg_is_in_list(void *data, const char *dev_name)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;

	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
		if (node && g_strcmp0(((mf_otg_node *)node)->name, dev_name) == 0)
			return true;
	}
	return false;
}

int mf_otg_insert_node(void *data, const char *name)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (mf_otg_is_in_list(ap, name)) {
		return MYFILE_ERR_NONE;
	} else {
		mf_otg_node *pNode = NULL;
		pNode = (mf_otg_node *)malloc(sizeof(mf_otg_node));

		if (pNode == NULL) {
			return MYFILE_ERR_ALLOCATE_MEMORY_FAIL;
		}
		memset(pNode, 0, sizeof(mf_otg_node));

		/* set name */
		pNode->name = g_strdup(name);
		/* set flag */
		pNode->flag_using = false;
		pNode->flag_editstart = false;

		ap->mf_FileOperation.otg_dev_list = eina_list_append(ap->mf_FileOperation.otg_dev_list, pNode);
	}
	return MYFILE_ERR_NONE;
}
int mf_otg_delete_node(void *data, const char *name)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (!mf_otg_is_in_list(ap, name)) {
		mf_debug();
		return MYFILE_ERR_NONE;
	} else {
		Eina_List *l = NULL;
		void *node = NULL;
		EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
			if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
				if (g_strcmp0(((mf_otg_node *)node)->name, name) == 0){
					free(((mf_otg_node *)node)->name);
					((mf_otg_node *)node)->name = NULL;
				}
				ap->mf_FileOperation.otg_dev_list = eina_list_remove_list(ap->mf_FileOperation.otg_dev_list, l);
				break;
			}
		}
	}
	return MYFILE_ERR_NONE;
}

/*
 *	flag_inc_dec:
 *		true:	insert node
 *		false:	delete node
*/
int mf_otg_list_update(void *data, bool flag_inc_dec)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	mf_debug();
	if (flag_inc_dec == TRUE) {
		DIR *pDir = NULL;
		struct dirent *ent;
		pDir = opendir(OTG_FOLDER);

		if (pDir == NULL) {
			return MYFILE_ERR_DIR_OPEN_FAIL;
		}

		while ((ent = readdir(pDir)) != NULL) {
			if (g_strcmp0(ent->d_name, ".") == 0 || g_strcmp0(ent->d_name, "..") == 0) {
				continue;
			}

			if ((ent->d_type & DT_DIR) == 0 && (ent->d_type & DT_REG) == 0) {
				mf_debug();
				continue;
			}

			mf_debug();
			char *test_name = g_strconcat(OTG_FOLDER, "/", ent->d_name, NULL);
			mf_debug("current test_name is %s", test_name);
			if (test_name != NULL) {
				mf_debug();
				/*we will check if the device is already in the list in mf_otg_insert_node*/
				if (flag_inc_dec == true) {
					mf_otg_insert_node(ap, test_name);
					mf_debug();
				}

				free(test_name);
				test_name = NULL;
			}
		}
		closedir(pDir);
	} else {
		Eina_List *l = NULL;
		void *node = NULL;

		EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
			if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
				mf_debug("name is %s", ((mf_otg_node *)node)->name);
				if (ecore_file_exists(((mf_otg_node *)node)->name) == false)
					mf_otg_delete_node(ap, ((mf_otg_node *)node)->name);
				}
			}
	}
	return MYFILE_ERR_NONE;
}

bool mf_otg_is_empty(void *data)
{
	return ecore_file_exists(OTG_FOLDER) && ecore_file_dir_is_empty(OTG_FOLDER);
}

char *mf_otg_get_node_in_use(void *data)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;

	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
		if ((mf_otg_node *)node != NULL) {
			if (((mf_otg_node *)node)->flag_using == true) {
				return ((mf_otg_node *)node)->name;
			}
		}
	}
	return NULL;
}

void mf_otg_set_node_using_flag(void *data, char *dev_name, bool status)
{
	assert(data);
	/*g_strcmp0 used, no need to check dev_name here*/
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;

	mf_debug("dev_name is %s", dev_name);
	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {

		if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
			if (g_strcmp0(((mf_otg_node *)node)->name, dev_name) == 0) {
				((mf_otg_node *)node)->flag_using = status;
			}
		}
	}
}

int mf_otg_get_node_using_flag(void *data, char *dev_name, bool *status)
{
	assert(data);
	assert(status);
	/*g_strcmp0 used, no need to check dev_name here*/
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;

	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
		if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
			if (g_strcmp0(((mf_otg_node *)node)->name, dev_name) == 0) {
				*status = ((mf_otg_node *)node)->flag_using;
				return MYFILE_ERR_NONE;
			}
		}
	}
	return MYFILE_ERR_GET_STAT_FAIL;
}

void mf_otg_set_node_editstart_flag(void *data, char *dev_name, bool status)
{
	assert(data);
	/*g_strcmp0 used, no need to check dev_name here*/
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;

	mf_debug("dev_name is %s", dev_name);
	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {

		if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
			if (g_strcmp0(((mf_otg_node *)node)->name, dev_name) == 0) {
				((mf_otg_node *)node)->flag_editstart = status;
			}
		}
	}
}

int mf_otg_get_node_editstart_flag(void *data, char *dev_name, bool *status)
{
	assert(data);
	assert(status);
	/*g_strcmp0 used, no need to check dev_name here*/
	struct appdata *ap = (struct appdata *)data;

	Eina_List *l = NULL;
	void *node = NULL;

	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {

		if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
			if (g_strcmp0(((mf_otg_node *)node)->name, dev_name) == 0) {
						*status = ((mf_otg_node *)node)->flag_editstart;
						return MYFILE_ERR_NONE;
			}
		}
	}
	return MYFILE_ERR_GET_STAT_FAIL;
}

void mf_otg_generate_removed_list(void *data)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	if (ap->mf_FileOperation.otg_dev_removed_list != NULL) {
		mf_debug();
		mf_otg_list_clean(ap, MF_OTG_LIST_REMOVED);
	}
	Eina_List *l = NULL;
	void *node = NULL;

	mf_otg_node *pNode = NULL;
	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {

		if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
			if (ecore_file_exists(((mf_otg_node *)node)->name) == false) {
				pNode = (mf_otg_node *) malloc(sizeof(mf_otg_node));

				if (pNode == NULL) {
					continue;
				}
				memset(pNode, 0, sizeof(mf_otg_node));
				/* set name */
				pNode->name = g_strdup(((mf_otg_node *)node)->name);
				/* set flag */
				pNode->flag_using = ((mf_otg_node *)node)->flag_using;
				pNode->flag_editstart = ((mf_otg_node *)node)->flag_editstart;

				ap->mf_FileOperation.otg_dev_removed_list = eina_list_append(ap->mf_FileOperation.otg_dev_removed_list, pNode);
			}
		}

	}
}

void mf_otg_clear_removed_device(void *data)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	const Eina_List *l = NULL;
	void *node = NULL;

	EINA_LIST_FOREACH(ap->mf_FileOperation.otg_dev_list, l, node) {
		if ((mf_otg_node *)node != NULL && ((mf_otg_node *)node)->name != NULL) {
			if (ecore_file_exists(((mf_otg_node *)node)->name) == false) {
				mf_otg_delete_node(ap, ((mf_otg_node *)node)->name);
			}
		}
	}
}

/*do external operation according to the opt*/
static void __mf_otg_dev_removed_ex_operation(void *data, mf_otg_ex_opt_e opt)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	/* try to remove all the popup and give an notice popup */
	if (opt & MF_OTG_OPT_REQ_POPUP_DEL) {	/*delete request popup in data transmit period*/
		ap->mf_MainWindow.pMmcRemovedPopup = mf_popup_create_popup(ap, POPMODE_TEXT_NOT_DISABLED, NULL,
									   "OTG device removed...", NULL, NULL, NULL, NULL, NULL);
	}

	if (opt & MF_OTG_OPT_NOR_POPUP_DEL) {	/*delete normal popup*/
		if (ap->mf_MainWindow.pNormalPopup) {
			evas_object_del(ap->mf_MainWindow.pNormalPopup);
			ap->mf_MainWindow.pNormalPopup = NULL;
		}
	}

}

void mf_otg_dev_all_removed_update(void *data, mf_otg_ex_opt_e opt)
{
	mf_debug();
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	mf_debug();
	Evas_Object *entry = NULL;
	myfileNaviBar *pNaviToShowStruct = NULL;
	myfileNaviBar *pNaviOTG = NULL;
	Evas_Object *pNaviBarStorageView = NULL;

	/*otg tab destroyed, finalize otg*/
	mf_otg_finalize(ap);

	mf_debug();

	pNaviBarStorageView = ap->mf_MainWindow.pNaviBar;
	pNaviOTG = mf_navi_bar_get_struct_by_label(ap, GET_STR(MF_LABEL_OTG));

	char status = MF_OTG_STATUS_NONE;

	if (pNaviOTG != NULL && pNaviOTG->naviFlagEditStart == TRUE)
		status |= MF_OTG_STATUS_EDIT;

	if (pNaviOTG != NULL && pNaviOTG->naviFlagInUse == TRUE)
		status |= MF_OTG_STATUS_USING;

	/*destroy otg navi bar*/
	mf_debug("navi count is %d", eina_list_count(ap->mf_MainWindow.plistNaviBar));
	mf_navi_bar_remove_list_item_by_label(ap, GET_STR(MF_LABEL_OTG));
	mf_debug();


	if (status & MF_OTG_STATUS_EDIT) {
		mf_navi_bar_recover_list(ap);
	}

	pNaviToShowStruct = mf_navi_bar_get_struct_by_label(ap, GET_SYS_STR(MF_LABEL_PHONE));

	if (status & MF_OTG_STATUS_EDIT) {
		ap->mf_Status.more = MORE_DEFAULT;
		mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
	} else if (status & MF_OTG_STATUS_USING){
		if ( ap->mf_Status.more == MORE_CREATE_FOLDER
		     && (ap->mf_Status.preMore == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.preMore == MORE_INTERNAL_COPY || ap->mf_Status.preMore == MORE_INTERNAL_MOVE)) {
			ap->mf_Status.more = ap->mf_Status.preMore;
		} else if (ap->mf_Status.more != MORE_INTERNAL_COPY_MOVE && ap->mf_Status.preMore != MORE_INTERNAL_COPY && ap->mf_Status.preMore != MORE_INTERNAL_MOVE) {
			ap->mf_Status.more = MORE_DEFAULT;
		}
		mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
	}

	/*delete external storage from tab bar*/
	mf_debug("navi count is %d", eina_list_count(ap->mf_MainWindow.plistNaviBar));
	/*check tab bar item to decide if delete tab bar*/
	mf_debug("navi count is %d", eina_list_count(ap->mf_MainWindow.plistNaviBar));

	/*clear some external detail after remove otg device*/
	mf_debug();
	if (status) {
		__mf_otg_dev_removed_ex_operation(ap, opt);
	} else {
		mf_otg_ex_opt_e updated_opt = opt;
		if (updated_opt & MF_OTG_OPT_NOR_POPUP_DEL)
			updated_opt ^= MF_OTG_OPT_NOR_POPUP_DEL;

		if (updated_opt & MF_OTG_OPT_REQ_POPUP_DEL)
			updated_opt ^= MF_OTG_OPT_REQ_POPUP_DEL;

		__mf_otg_dev_removed_ex_operation(ap, updated_opt);
	}

	if (ap->mf_Status.more == MORE_RENAME) {
		entry = ap->mf_MainWindow.pEntry;
		if (entry != NULL) {
			elm_object_focus_set(entry, EINA_TRUE);
		}
	}
}

void mf_otg_dev_removed_update(void *data, mf_otg_ex_opt_e opt)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	Evas_Object *pNaviBarStorageView = NULL;
	myfileNaviBar *pNavi_s = NULL;
	myfileNaviBar *pNaviToShowStruct = NULL;

	pNaviBarStorageView = ap->mf_MainWindow.pNaviBar;
	pNavi_s = mf_navi_bar_get_struct_by_label(ap, GET_STR(MF_LABEL_OTG));
	if (pNavi_s == NULL) {
		return;
	}
	Eina_List *l = NULL;
	void *node = NULL;

	int error_code = 0;
	char status = MF_OTG_STATUS_NONE;
	EINA_LIST_FOREACH (ap->mf_FileOperation.otg_dev_removed_list, l, node) {
		if (node) {
			bool using_status = FALSE;
			error_code = mf_otg_get_node_using_flag(ap, ((mf_otg_node *)node)->name, &using_status);
			if (error_code == 0 && using_status == TRUE)
				status |= MF_OTG_STATUS_USING;
			bool edit_status = FALSE;
			error_code = mf_otg_get_node_editstart_flag(ap, ((mf_otg_node *)node)->name, &edit_status);
			if (error_code == 0 && edit_status == TRUE)
				status |= MF_OTG_STATUS_EDIT;
		}
	}

	/*check both flags*/
	mf_debug("status is %d", status);
	mf_debug("status & MF_OTG_STATUS_EDIT is %d", status & MF_OTG_STATUS_EDIT);

	if (status & MF_OTG_STATUS_EDIT) {
		mf_debug();
		/*pop edit related view for each navi bar*/
		pNaviToShowStruct = mf_navi_bar_recover_list(ap);

		/*pop to the root view of OTG*/
		Elm_Object_Item *bottom_it = elm_naviframe_bottom_item_get(pNaviBarStorageView);

		elm_naviframe_item_pop_to(bottom_it);

		/* set the navi's currentpath to be root path*/
		if (pNavi_s->pCurrentPath != NULL) {
			free(pNavi_s->pCurrentPath);
		}
		pNavi_s->pCurrentPath = g_strdup(OTG_FOLDER);
		ap->mf_MainWindow.pNaviLayout = ap->mf_Otg.rootpath_layout;
		ap->mf_MainWindow.pNaviBox = ap->mf_Otg.rootpath_box;

		if (pNavi_s->naviFlagInUse == TRUE){
			if (ap->mf_Status.path != NULL) {
				g_string_free(ap->mf_Status.path, TRUE);
			}
			ap->mf_Status.path = g_string_new(OTG_FOLDER);
		}
	}else if (status & MF_OTG_STATUS_USING) {
		if (ap->mf_Status.more == MORE_SEARCH) {
			if (ap->mf_FileOperation.sync_pipe != NULL) {
				ecore_pipe_del(ap->mf_FileOperation.sync_pipe);
				ap->mf_FileOperation.sync_pipe = NULL;
			}

			if (ap->mf_FileOperation.search_IME_hide_timer != NULL) {
				ecore_timer_del(ap->mf_FileOperation.search_IME_hide_timer);
				ap->mf_FileOperation.search_IME_hide_timer = NULL;
			}

			if (ap->mf_Status.search_handler > 0) {
				mf_search_stop(ap->mf_Status.search_handler);
			}

			if (ap->mf_Status.search_handler > 0) {
				mf_search_finalize(&ap->mf_Status.search_handler);
			}
		}
		if (ap->mf_Status.more == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE
		    || (ap->mf_Status.more == MORE_CREATE_FOLDER && (ap->mf_Status.preMore == MORE_INTERNAL_COPY_MOVE || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE))) {
			/*pop unused path select view*/
			/*create and push root view of OTG*/
			SAFE_FREE_GSTRING(ap->mf_Status.path);
			ap->mf_Status.path = g_string_new(OTG_FOLDER);

			SAFE_FREE_CHAR(pNavi_s->pCurrentPath);
			pNavi_s->pCurrentPath = g_strdup(OTG_FOLDER);

			if (ap->mf_Status.more == MORE_CREATE_FOLDER)
				ap->mf_Status.more = ap->mf_Status.preMore;

			mf_navi_bar_create_path_select_view_otg_root(ap);
			mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
		} else {
			ap->mf_Status.more = MORE_DEFAULT;

			/*pop to the root view of OTG*/
			Elm_Object_Item *bottom_it = elm_naviframe_bottom_item_get(pNaviBarStorageView);

			elm_naviframe_item_pop_to(bottom_it);

			/* set the navi's currentpath to be root path*/
			if (pNavi_s->pCurrentPath != NULL) {
				free(pNavi_s->pCurrentPath);
			}
			pNavi_s->pCurrentPath = g_strdup(OTG_FOLDER);
			ap->mf_MainWindow.pNaviLayout = ap->mf_Otg.rootpath_layout;
			ap->mf_MainWindow.pNaviBox = ap->mf_Otg.rootpath_box;
			ap->mf_MainWindow.pNaviItem = ap->mf_Otg.rootpath_navi_item;

			if (pNavi_s->naviFlagInUse == TRUE){
				if (ap->mf_Status.path != NULL) {
					g_string_free(ap->mf_Status.path, TRUE);
				}
				ap->mf_Status.path = g_string_new(OTG_FOLDER);
			}
		}
	}

	if ((status & MF_OTG_STATUS_EDIT)
	    || (pNavi_s && pNavi_s->naviFlagInUse && ((status & MF_OTG_STATUS_USING) || mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str) == MYFILE_OTG))) {
		mf_debug("status is %d", status);
		/*refresh the list*/
		mf_debug("status & MF_OTG_STATUS_EDIT is %d", status & MF_OTG_STATUS_EDIT);
		if (status & MF_OTG_STATUS_USING) {
			mf_debug();
			pNaviToShowStruct = pNavi_s;

			if ( ap->mf_Status.more == MORE_CREATE_FOLDER && (ap->mf_Status.preMore == MORE_INTERNAL_COPY_MOVE
			     || ap->mf_Status.more == MORE_INTERNAL_COPY || ap->mf_Status.more == MORE_INTERNAL_MOVE)) {
				ap->mf_Status.more = ap->mf_Status.preMore;
			} else if (ap->mf_Status.more != MORE_INTERNAL_COPY_MOVE && ap->mf_Status.more != MORE_INTERNAL_COPY && ap->mf_Status.more != MORE_INTERNAL_MOVE) {
				ap->mf_Status.more = MORE_DEFAULT;
			}
		}
		mf_navi_bar_refresh_recovered_view(ap, pNaviToShowStruct);
	}

	/*clear some external detail after remove otg device*/
	mf_debug();
	if (status) {
		__mf_otg_dev_removed_ex_operation(ap, opt);
	} else {
		mf_otg_ex_opt_e updated_opt = opt;
		if (updated_opt & MF_OTG_OPT_NOR_POPUP_DEL)
			updated_opt ^= MF_OTG_OPT_NOR_POPUP_DEL;

		if (updated_opt & MF_OTG_OPT_REQ_POPUP_DEL)
			updated_opt ^= MF_OTG_OPT_REQ_POPUP_DEL;

		__mf_otg_dev_removed_ex_operation(ap, updated_opt);
	}
	mf_debug("dev list count is %d", eina_list_count(ap->mf_FileOperation.otg_dev_list));
	mf_otg_list_update(ap, FALSE);
	mf_debug("dev list count is %d", eina_list_count(ap->mf_FileOperation.otg_dev_list));

}
void mf_otg_finalize(void *data)
{
	assert(data);
	struct appdata *ap = (struct appdata *)data;

	mf_otg_list_clean(ap, MF_OTG_LIST_DEV);
	mf_otg_list_clean(ap, MF_OTG_LIST_REMOVED);
}

#endif
