/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <nfc.h>
#include <app_service.h>
#include <vconf.h>

#include "mf-s-beam.h"
#include "mf-dlog.h"
#include "mf-util.h"

#define S_BEAM_SEPERATOR	"|"

typedef struct {
	bool initialized;
	bool enabled;

	Mf_Sbeam_Cb callback;
	void *user_data;
} mf_sbeam_s;

static mf_sbeam_s *g_sbeam_data = NULL;

bool __mf_sbeam_is_on()
{
	bool ready = false;

	int value = 0;
	if (vconf_get_bool(VCONFKEY_NFC_SBEAM, &value) == 0) {
		if (value == VCONFKEY_NFC_SBEAM_ON)
			ready = true;
		else
			mf_debug("sbeam is off");
	} else {
		mf_error("vconf_get_int()");
	}

	return ready;
}

static void
__mf_sbeam_target_discovered_cb(nfc_discovered_type_e type, nfc_p2p_target_h target, void *user_data)
{
	MF_TRACE_BEGIN;
	MF_CHECK(g_sbeam_data);
	MF_CHECK(g_sbeam_data->enabled);

	if (type == NFC_DISCOVERED_TYPE_ATTACHED && g_sbeam_data->callback)
	{
		mf_debug("NFC_DISCOVERED_TYPE_ATTACHED");
		g_sbeam_data->callback(g_sbeam_data->user_data);
	}
	else if (type == NFC_DISCOVERED_TYPE_DETACHED)
	{
		mf_debug("NFC_DISCOVERED_TYPE_DETACHED");
	}
}


static bool
__mf_sbeam_nfc_mgr_init()
{
	MF_CHECK_FALSE(g_sbeam_data);

	int ret = 0;
	if (!g_sbeam_data->initialized) {
		ret = nfc_manager_initialize(NULL, NULL);
		if (ret == NFC_ERROR_NONE) {
			mf_debug("NFC initializing is success. activating");
			g_sbeam_data->initialized = true;
		} else {
			mf_error("nfc_manager_initialize().. [0x%x]");
			return false;
		}
	}

	ret = nfc_manager_set_p2p_target_discovered_cb(__mf_sbeam_target_discovered_cb, g_sbeam_data);
	if (ret != NFC_ERROR_NONE) {
		mf_debug("nfc_manager_set_p2p_target_discovered_cb()..[0x%x]", ret);
		return false;
	}

	return true;
}

static bool
__mf_sbeam_nfc_mgr_deinit()
{
	MF_CHECK_FALSE(g_sbeam_data);

	int ret = 0;
	if (g_sbeam_data->initialized) {
		ret = nfc_manager_deinitialize();
		if (ret != NFC_ERROR_NONE) {
			mf_error("nfc_manager_deinitialize().. [0x%x]", ret);
			return false;
		}
		g_sbeam_data->initialized = false;
	}

	return true;
}

static void
__mf_sbeam_state_changed_cb(keynode_t *node, void *user_data)
{
	MF_TRACE_BEGIN;
	mf_sbeam_s *sbeam_data = user_data;
	MF_CHECK(sbeam_data);

	if (__mf_sbeam_is_on() && sbeam_data->enabled)
		__mf_sbeam_nfc_mgr_init();
	else
		__mf_sbeam_nfc_mgr_deinit();
}

bool
mf_sbeam_init()
{
	MF_TRACE_BEGIN;
	g_sbeam_data = calloc(1, sizeof(mf_sbeam_s));
	MF_CHECK_VAL(g_sbeam_data, false);

	if (vconf_notify_key_changed(VCONFKEY_NFC_SBEAM, __mf_sbeam_state_changed_cb, g_sbeam_data) < 0) {
		mf_error("fail to vconf_notify_key_changed()");
		mf_sbeam_finalize();
		return false;
	}

	if (__mf_sbeam_is_on())
		mf_sbeam_enable();

	return true;
}

void
mf_sbeam_finalize()
{
	MF_TRACE_BEGIN;
	mf_sbeam_disable();
	SAFE_FREE_CHAR(g_sbeam_data);
}

bool
mf_sbeam_enable()
{
	MF_TRACE_BEGIN;
	MF_CHECK_FALSE(g_sbeam_data);

	g_sbeam_data->enabled = true;

	return __mf_sbeam_nfc_mgr_init();
}
bool mf_sbeam_disable()
{
	MF_TRACE_BEGIN;
	MF_CHECK_FALSE(g_sbeam_data);

	g_sbeam_data->enabled = false;

	return __mf_sbeam_nfc_mgr_deinit();
}

bool mf_sbeam_set_callback(Mf_Sbeam_Cb callback, void *user_data)
{
	MF_TRACE_BEGIN;
	MF_CHECK_FALSE(g_sbeam_data);

	g_sbeam_data->callback = callback;
	g_sbeam_data->user_data = user_data;

	return true;
}

bool mf_sbeam_share_files(Evas_Object *win, Eina_List *file_list)
{
	MF_CHECK_FALSE(win);

	int ret = 0;
	service_h svc_handle;
	ret = service_create(&svc_handle);
	if (ret != SERVICE_ERROR_NONE) {
		mf_error("service_create().. [0x%x]", ret);
		goto exception;
	}

	ret = service_set_operation(svc_handle, "http://tizen.org/appcontrol/operation/nfc_sbeam_send");
	if (ret != SERVICE_ERROR_NONE) {
		mf_error("service_set_operation().. [0x%x]", ret);
		goto exception;
	}

	ret = service_set_mime(svc_handle, "text/DirectShareFile");
	if (ret != SERVICE_ERROR_NONE) {
		mf_error("service_set_mime().. [0x%x]", ret);
		goto exception;
	}
	ret = service_set_window(svc_handle, elm_win_xwindow_get(win));
	if (ret != SERVICE_ERROR_NONE) {
		mf_error("service_set_window().. [0x%x]", ret);
		goto exception;
	}

	char *uri = NULL;
	if (file_list) {
		Eina_List *current = file_list;
		Eina_List *l = NULL;
		GString *pNode = NULL;


		EINA_LIST_FOREACH(current, l, pNode) {
			if (pNode != NULL) {
				if (uri == NULL) {
					uri = g_strconcat(pNode->str, NULL);
				} else {
					gchar *temp = uri;
					uri = g_strconcat(uri, S_BEAM_SEPERATOR, pNode->str, NULL);
					g_free(temp);
				}
			}

		}
	}
	mf_debug("files = [%s]", uri);
	ret = service_set_uri(svc_handle, uri);
	if (ret != SERVICE_ERROR_NONE) {
		mf_error("service_set_uri().. [0x%x]", ret);
		goto exception;
	}
	SAFE_FREE_CHAR(uri);


	ret = service_send_launch_request(svc_handle, NULL, NULL);
	if (ret != SERVICE_ERROR_NONE) {
		mf_error("service_send_launch_request().. [0x%x]", ret);
		goto exception;
	}

	service_destroy(svc_handle);

	return true;

exception:
	service_destroy(svc_handle);
	svc_handle = NULL;

	return false;
}

