/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <Eina.h>
#include <Evas.h>
#include <Ethumb.h>
#include <assert.h>

#include "mf-main.h"
#include "mf-widget.h"
#include "mf-util.h"
#include "mf-fs-util.h"
#include "mf-launch.h"
#include "mf-drm.h"
#include "mf-share.h"
#include "mf-resource.h"

int mf_share_mode_get(Eina_List * selected_list)
{
	mf_retvm_if(selected_list == NULL, -1, "selected_list is NULL");
	int file_mode = SHARE_FILE_MODE_NORMAL;

	Eina_List *l = NULL;
	GString *pNode = NULL;
	int image_count = 0;
	int video_count = 0;
	EINA_LIST_FOREACH(selected_list, l, pNode) {
		if (pNode != NULL) {
			char *file_path = pNode->str;
			if (ecore_file_is_dir(file_path)) {
				return 0;
			}

			fsFileType category = FILE_TYPE_NONE;

			mf_file_attr_get_file_category(file_path, &category);
			if (category == FILE_TYPE_IMAGE) {
				image_count++;
				if (image_count > 1) {
					file_mode = file_mode | SHARE_FILE_MODE_MULTI_IMAGE;
				} else {
					file_mode = file_mode | SHARE_FILE_MODE_IMAGE;
				}
			} else if (category == FILE_TYPE_VIDEO) {
				video_count++;
				if (video_count > 1) {
					file_mode = file_mode | SHARE_FILE_MODE_MULTI_VIDEO;
				} else {
					file_mode = file_mode | SHARE_FILE_MODE_VIDEO;
				}
			} else {
				file_mode = file_mode | SHARE_FILE_MODE_OTHERS;
			}
		}
	}
	return file_mode;
}
static int __mf_share_get_selected_files(void *data, gchar ** selected_files, Eina_List * selected_list)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ap == NULL, -1, "ap is NULL");
	mf_retvm_if(selected_list == NULL, 0, "selected_list is NULL");
	mf_retvm_if(selected_files == NULL, -1, "selected_files is NULL");

	int file_count = 0;
	Eina_List *l = NULL;
	GString *pNode = NULL;
	gchar *message = NULL;

	EINA_LIST_FOREACH(ap->mf_FileRecordList.selected_files, l, pNode) {
		if (pNode != NULL) {
			if (!ecore_file_is_dir(pNode->str) && mf_drm_is_drm_file(pNode->str) == 0) {
				if (mf_drm_is_converted_fl(pNode->str) == 0) {
					message = g_strconcat(GET_STR(MF_POPUP_MESSAGE_SEND_FAIL1), "<br>", ecore_file_file_get(pNode->str),
							GET_STR(MF_POPUP_MESSAGE_SEND_FAIL2), NULL);
					ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message,
											 NULL, NULL, NULL, NULL, NULL);

					g_free(message);
					message = NULL;
					if (*selected_files != NULL) {
						  g_free(*selected_files);
						  *selected_files = NULL;
					}
					MF_TRACE_END;
					return -1;
				}
			} else if (ecore_file_is_dir(pNode->str)) {
				message = GET_STR(MF_POPUP_MESSAGE_SEND_FAIL3);
				ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);

				if (*selected_files != NULL) {
					g_free(*selected_files);
					*selected_files = NULL;
				}
				MF_TRACE_END;
				return -1;
			}

			if (*selected_files == NULL) {
				*selected_files = g_strconcat(pNode->str, NULL);
				file_count++;
			} else {
				gchar *temp = *selected_files;
				if (ap->mf_Status.more == MORE_SENDVIABLUETOOTH
				) {
					*selected_files = g_strconcat(*selected_files, "?", pNode->str, NULL);
					file_count++;
				}
				else {
					*selected_files = g_strconcat(*selected_files, "\n", pNode->str, NULL);
					file_count++;
				}

				g_free(temp);
			}
		}
	}
	MF_TRACE_END;
	return file_count;
}

void mf_share_launch_multi_file(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);
	gchar *myFilenames = NULL;
	int number_of_files = 0;
	char count_arg[20] = { 0, };
	const char *message = NULL;

	number_of_files = __mf_share_get_selected_files(ap, &myFilenames, ap->mf_FileRecordList.selected_files);
	if (number_of_files < 0) {
		MF_TRACE_END;
		return;
	} else if (number_of_files == 0) {
		message = GET_SYS_STR(MF_LABEL_NOTHING_SELECTED);
		ap->mf_MainWindow.pNormalPopup = mf_popup_create_popup(ap, POPMODE_TEXT, NULL, message, NULL, NULL, NULL, NULL, NULL);
		MF_TRACE_END;
		return;
	} else {
		snprintf(count_arg, sizeof(count_arg), "%d", number_of_files);
		if (ap->mf_Status.more == MORE_SENDVIABLUETOOTH) {
			mf_debug("::::::::::Send via bluetooth:::::::::::::");
			mf_launch_load_ug(ap, myFilenames, MF_LOAD_UG_BLUETOOTH, count_arg);
		} else if (ap->mf_Status.more == MORE_SENDVIAEMAIL) {
			mf_debug("::::::::::Send via email::::::::::::: files is [%s]", myFilenames);
			mf_launch_load_ug(ap, myFilenames, MF_LOAD_UG_EMAIL, count_arg);
		} else if (ap->mf_Status.more == MORE_SENDVIAMESSAGE) {
			mf_debug("::::::::::Send via message::::::::::::: files is [%s]", myFilenames);
			mf_launch_load_ug(ap, myFilenames, MF_LOAD_UG_MESSAGE, count_arg);
		}

		g_free(myFilenames);
	}
	MF_TRACE_END;
}

