/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */




#include <stdio.h>
#include <sys/time.h>
#include <glib.h>
#include <vconf.h>
#include <media_content.h>
#include <app.h>
#include <Elementary.h>
#ifdef MYFILE_MOTION_FEATURE
#include "mf-sensor.h"
#endif


#include "mf-main.h"
#include "mf-conf.h"
#include "mf-dlog.h"
#include "mf-util.h"
#include "mf-callback.h"
#include "mf-widget.h"
#include "mf-gadget.h"
#include "mf-fm-svc-wrapper.h"
#include "mf-ta.h"
#include "mf-log.h"
#include "mf-otg.h"
#include "mf-launch.h"
#include "mf-language-mgr.h"
#include "mf-s-beam.h"

#define MF_B_KEY_PATH "path"

GString *phone_folder_as_param;

static bool __mf_main_create_app(void *data);
static void __mf_main_terminate_app(void *data);
static void __mf_main_stop_app(void *data);
static void __mf_main_resume_app(void *data);
static void __mf_main_reset_app(service_h service, void *data);
static void __mf_main_storage_status_get(void *data);

static void __mf_main_data_init(void *data)
{
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");

	struct appdata *ap = (struct appdata *)data;
	/*set state value for create content*/
	ap->mf_Status.more = MORE_DEFAULT;
	ap->mf_Status.path = g_string_new(PHONE_FOLDER);
	ap->mf_SharedGadget.ug = NULL;
	ap->mf_Status.iStorageState = MYFILE_PHONE;
	ap->mf_Status.iFolderSortType = MYFILE_SORT_BY_DATE_R2O;
	ap->mf_Status.flagLCDLock = EINA_FALSE;
	ap->mf_Status.rotation_type = MF_ROTATE_PORTRAIT;
	ap->mf_Status.flagNaviPush = EINA_FALSE;

	/* region format related */
	ap->mf_Status.flagIcuInit = FALSE;
	ap->mf_Status.generator = NULL;
	ap->mf_Status.formatter = NULL;
	ap->mf_Status.flagRootView = EINA_TRUE;
	ap->mf_Status.flagIME = EINA_TRUE;
	ap->mf_Status.view_type = mf_view_root;
	ap->mf_Status.preViewType = mf_view_root;
	mf_util_get_vconf_value(VCONF_TYPE_NFC_ENABLE, &ap->mf_Status.iNFCState);
	mf_util_get_vconf_value(VCONF_TYPE_VIEW_STYLE, &ap->mf_Status.flagViewType);
	mf_util_get_vconf_value(VCONF_TYPE_EXTENSION_STATE, &ap->mf_Status.iExtensionState);

	MF_TRACE_END;

}

/******************************
** Prototype    : mf_main_load_edj
** Description  :
** Input        : Evas_Object *parent
**                const char *file
**                const char *group
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
Evas_Object *mf_main_load_edj(Evas_Object * parent, const char *file, const char *group)
{
	mf_debug();
	Evas_Object *eo;
	int r;

	eo = elm_layout_add(parent);
	elm_object_focus_set(eo, EINA_FALSE);
	if (eo) {
		r = elm_layout_file_set(eo, file, group);
		if (!r) {
			evas_object_del(eo);
			return NULL;
		}

		evas_object_size_hint_weight_set(eo, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	}

	return eo;
}

#ifdef MYFILE_MOTION_FEATURE
static void __mf_app_doubletap_cb(unsigned long long timestamp, void *data)
{
	MF_TRACE_BEGIN;
	mf_widget_first_item_bring_in(data);
}
#endif

/******************************
** Prototype    : __mf_main_del_win
** Description  :
** Input        : void *data
**                Evas_Object *obj
**                void *event
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_main_del_win(void *data, Evas_Object * obj, void *event)
{
	elm_exit();
}

/******************************
** Prototype    : __mf_main_create_win
** Description  :
** Input        : const char *name
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static Evas_Object *__mf_main_create_win(const char *name)
{
	mf_debug();
	Evas_Object *eo;
	int w, h;
	eo = elm_win_add(NULL, name, ELM_WIN_BASIC);
	elm_object_focus_set(eo, EINA_FALSE);
	elm_win_autodel_set(eo, 1);
	if (eo) {
		elm_win_title_set(eo, name);
		evas_object_smart_callback_add(eo, "delete,request", __mf_main_del_win, NULL);
		ecore_x_window_size_get(ecore_x_window_root_first_get(), &w, &h);
		evas_object_resize(eo, w, h);
	}
	if (eo) {
		evas_object_show(eo);
		return eo;
	} else {
		return NULL;
	}
}


static Evas_Object *__mf_main_create_bg(Evas_Object *win)
{
	mf_retv_if(win == NULL, NULL);
	Evas_Object *bg = elm_bg_add(win);
	elm_object_focus_set(bg, EINA_FALSE);

	evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	elm_win_resize_object_add(win, bg);

	evas_object_show(bg);

	return bg;
}


/******************************
** Prototype    : __mf_main_capture_idle_img
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/

static int __mf_main_check_path_vaild(const char *dir, int *dir_type)
{
	mf_retvm_if(dir == NULL, -1, "dir is NULL");
	mf_retvm_if(dir_type == NULL, -1, "state is NULL");

	if (g_file_test(dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)
	    && strncmp(dir, ".", 1)) {
		if (strncmp(dir, PHONE_FOLDER, strlen(PHONE_FOLDER)) == 0) {
			*dir_type = MYFILE_PHONE;
		} else if (strncmp(dir, MEMORY_FOLDER, strlen(MEMORY_FOLDER)) == 0) {
			*dir_type = MYFILE_MMC;
#ifdef MYFILE_USB_OTG
		} else if (strncmp(dir, OTG_FOLDER, strlen(OTG_FOLDER)) == 0){
			*dir_type = MYFILE_OTG;
#endif
		}else {
			*dir_type = MYFILE_NONE;
		}
	} else {
		*dir_type = MYFILE_NONE;
	}
	return 0;
}

static void __mf_main_bundle_parse(service_h service, void *data)
{
	MF_TRACE_BEGIN;
	assert(data);

	struct appdata *ap = (struct appdata *)data;

	service_get_extra_data (service, "path", &ap->mf_Bundle.path);
	service_get_extra_data (service, "select_type", &ap->mf_Bundle.select_type);
	service_get_extra_data (service, "file_type", &ap->mf_Bundle.file_type);
	service_get_extra_data (service, "marked_mode", &ap->mf_Bundle.marked_mode);
	service_get_extra_data (service, "drm_type", &ap->mf_Bundle.drm_type);
	MF_TRACE_END;
}

static void __mf_main_view_create(void *data)
{
	MF_TRACE_BEGIN;
	assert(data);
	MF_TA_ACUM_ITEM_BEGIN("        mf_main_view_create", 0);
	struct appdata *ap = (struct appdata *)data;
	//MF_TA_ACUM_ITEM_END("  create_to_reset", 0);

	//create phone navibar struct

	MF_TA_ACUM_ITEM_BEGIN("    mf_widget_refresh_view", 0);
	mf_widget_refresh_view(ap);
	MF_TA_ACUM_ITEM_END("    mf_widget_refresh_view", 0);
	MF_TA_ACUM_ITEM_END("        mf_main_view_create", 0);

}

Eina_Bool __mf_main_prop_change(void *data, int ev_type, void *ev)
{

	MF_TRACE_BEGIN;
	assert(data);
	Ecore_X_Event_Window_Property *event = ev;

	if (event->win != ecore_x_window_root_first_get())
		return ECORE_CALLBACK_PASS_ON;

	if (event->atom != ecore_x_atom_get("FONT_TYPE_change")) {
		return ECORE_CALLBACK_PASS_ON;
	}

	return ECORE_CALLBACK_PASS_ON;
}

static Eina_Bool
__mf_main_app_init_idler_cb(void *data)
{
	MF_TRACE_BEGIN;
	assert(data);
	struct appdata *ap = (struct appdata *)data;
	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_app_init_idler_cb", 0);


	SAFE_FREE_ECORE_EVENT(ap->mf_MainWindow.font_event);

	ap->mf_MainWindow.font_event = ecore_event_handler_add(ECORE_X_EVENT_WINDOW_PROPERTY,
								__mf_main_prop_change,
								ap);

	SAFE_FREE_ECORE_EVENT(ap->mf_MainWindow.event);
	ap->mf_MainWindow.event = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN,
				(Ecore_Event_Handler_Cb)mousedown_cb, ap);

	//MF_TA_ACUM_ITEM_BEGIN("    mmc callback set", 0);
	mf_callback_set_mmc_state_cb(ap);
	//MF_TA_ACUM_ITEM_END("    mmc callback set", 0);
#ifdef MYFILE_USB_OTG
	//MF_TA_ACUM_ITEM_BEGIN("    otg callback set", 0);
	mf_callback_set_otg_state_cb(ap);
	//MF_TA_ACUM_ITEM_END("    otg callback set", 0);
#endif
	ap->mf_Status.app_init_idler = NULL;

	if (mf_sbeam_init()) {
		mf_sbeam_set_callback(mf_callback_sbeam_cb, ap);
	}

#ifdef MYFILE_MOTION_FEATURE
	if(!mf_sensor_init()){
		if(!mf_sensor_set_callback(MF_MOTION_DOUBLETAP, __mf_app_doubletap_cb, ap)){
			mf_sensor_start(MF_MOTION_DOUBLETAP);
		}
	}
	ap->mf_Status.doubletab_event_handler = mf_callback_motion_double_tap;
#endif
	efreet_mime_init();

	MF_TA_ACUM_ITEM_END("    __mf_main_app_init_idler_cb", 0);

	return ECORE_CALLBACK_CANCEL;
}

static int __mf_main_remake_app(service_h service, void *data)
{
	MF_TRACE_BEGIN;
	assert(data);
	MF_TA_ACUM_ITEM_BEGIN("      mf_main_remake_app", 0);
	struct appdata *ap = (struct appdata *)data;
	MF_TA_ACUM_ITEM_BEGIN("      check mass storage state", 0);

	/*if myfile state is not normal,  it is not handled argument*/
	if (ap->mf_Status.more != MORE_DEFAULT) {
		mf_warnig("Fail to handle budle, current myfiles stat is %d", ap->mf_Status.more);
		goto RAISE_WIN;
	}
	MF_TA_ACUM_ITEM_END("      check mass storage state", 0);
	MF_TA_ACUM_ITEM_BEGIN("      __mf_main_view_create", 0);

	__mf_main_view_create(ap);
	MF_TA_ACUM_ITEM_END("      __mf_main_view_create", 0);

RAISE_WIN:

/**************pre-condition test to launch myfile app******************/
	MF_TA_ACUM_ITEM_END("      mf_main_remake_app", 0);

	MF_TA_ACUM_ITEM_END(" Myfiles Launch Time", 0);
	return 0;

}


/*	system operation callbacks	*/
/******************************
** Prototype    : _keydown_event
** Description  :
** Input        : void *data
**                int type
**                void *event
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
#define TEMP_ENDKEY_ROLLBACK
#if 0
static Eina_Bool __mf_main_keydown_event_callback(void *data, int type, void *event)
{
	Ecore_Event_Key *ev = event;
	struct appdata *ap = (struct appdata *)data;
	mf_retvm_if(ev == NULL, ECORE_CALLBACK_RENEW, "Ecore_Event_Key is NULL");
	mf_retvm_if(ap == NULL, ECORE_CALLBACK_RENEW, "appdata is NULL");

	if (!strcmp(ev->keyname, KEY_END)) {
#ifdef TEMP_ENDKEY_ROLLBACK
		if (ap->mf_Status.more != MORE_DEFAULT) {
			if (ap->mf_MainWindow.pProgressPopup != NULL) {
				if (ap->mf_Status.more == MORE_DATA_COPYING
				    || ap->mf_Status.more == MORE_DATA_MOVING
				    || ap->mf_Status.more == MORE_DELETE || ap->mf_Status.more == MORE_IDLE_DELETE) {
					mf_callback_progress_bar_cancel_cb(ap, NULL, NULL);
				}
			}
		}
		if (ap->mf_FileOperation.search_IME_hide_timer != NULL) {
			ecore_timer_del(ap->mf_FileOperation.search_IME_hide_timer);
			ap->mf_FileOperation.search_IME_hide_timer = NULL;
		}
		elm_exit();
#else
		mf_debug();
		if (ap->mf_SharedGadget.ug != NULL) {
			mf_debug();
			ug_send_key_event(UG_KEY_EVENT_END);
		} else {
			if (ap->mf_MainWindow.pContextPopup != NULL) {
				mf_debug();
				evas_object_del(ap->mf_MainWindow.pContextPopup);
				ap->mf_MainWindow.pContextPopup = NULL;
			} else {
				if (ap->mf_Status.more == MORE_DEFAULT) {
					mf_debug();
					if (mf_fm_svc_wrapper_is_root_path(ap->mf_Status.path->str)) {
						elm_exit();
					} else {
						mf_callback_upper_folder_cb(ap);
					}
				} else {
					if (ap->mf_MainWindow.pNormalPopup == NULL) {
						mf_callback_cancel_cb(ap, NULL, NULL);
					} else {
						if (ap->mf_Status.more ==
						    MORE_DATA_COPYING
						    || ap->mf_Status.more ==
						    MORE_DATA_MOVING || ap->mf_Status.more == MORE_DELETE || ap->mf_Status.more == MORE_IDLE_DELETE) {
							mf_callback_progress_bar_cancel_cb(ap, NULL, NULL);
						}
					}
				}
			}

		}
#endif
	}
	return ECORE_CALLBACK_RENEW;
}
#endif
/******************************
** Prototype    : __mf_main_create_app
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_main_storage_status_get(void *data)
{
	mf_retm_if(data == NULL, "data is NULL");
	struct appdata *ap = (struct appdata*)data;
	int mmc_card = 0;
	int error_code = 0;

	MF_TA_ACUM_ITEM_BEGIN("      storage check", 0);
	error_code = mf_util_is_mmc_on(&mmc_card);
	if (error_code == 0 && mmc_card == 1) {
		ap->mf_Status.iStorageState |= MYFILE_MMC;
	}
	/*check if usb otg mounted */
#ifdef MYFILE_USB_OTG
	int otg = 0;
	error_code = mf_util_is_otg_on(&otg);
	mf_debug("error code is %d, org is %d", error_code, otg);
	if (error_code == 0 && otg == 1) {
		mf_debug();
		ap->mf_Status.iStorageState |= MYFILE_OTG;
	}
#endif
	MF_TA_ACUM_ITEM_END("      storage check", 0);
}

bool __mf_main_create_app(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	int ret = 0;

	MF_TA_ACUM_ITEM_END("  main-to-create", 0);


	MF_TA_ACUM_ITEM_BEGIN("    g_thread_init", 0);
	if (!g_thread_supported()) {
		g_thread_init(NULL);
	}
	MF_TA_ACUM_ITEM_END("    g_thread_init", 0);
	MF_TA_ACUM_ITEM_BEGIN("  __mf_main_create_app", 0);

	MF_TA_ACUM_ITEM_BEGIN("    mf_language_mgr_create", 0);
	mf_language_mgr_create();
	MF_TA_ACUM_ITEM_END("    mf_language_mgr_create", 0);


	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_data_init", 0);
	__mf_main_data_init(ap);
	MF_TA_ACUM_ITEM_END("    __mf_main_data_init", 0);

	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_create_win", 0);

	ap->mf_MainWindow.pWindow = __mf_main_create_win(PACKAGE);
	mf_retvm_if(ap->mf_MainWindow.pWindow == NULL, -1, "Fail to __mf_main_create_win()");

	evas_object_geometry_get(ap->mf_MainWindow.pWindow, NULL, NULL, &ap->mf_MainWindow.root_w, &ap->mf_MainWindow.root_h);

	MF_TA_ACUM_ITEM_END("    __mf_main_create_win", 0);

#ifdef MYFILE_SPLIT_VIEW
	if(mf_util_is_rotation_lock() == 0) {
		mf_debug("rotation is locked");
		ap->mf_Status.rotation_type = MF_ROTATE_PORTRAIT;
		ap->mf_Status.rotation_angle = 0;
	} else {
		app_device_orientation_e  rotate_mode;

		rotate_mode = app_get_device_orientation();

		mf_util_rotate_state_set(ap, rotate_mode);
	}


#endif

	/**************start to launch myfile app******************/


	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_create_bg", 0);
	ap->mf_MainWindow.pBackGround = __mf_main_create_bg(ap->mf_MainWindow.pWindow);
	MF_TA_ACUM_ITEM_END("    __mf_main_create_bg", 0);

	ap->mf_MainWindow.pConformant = mf_widget_create_conform(ap->mf_MainWindow.pWindow);
	elm_win_resize_object_add(ap->mf_MainWindow.pWindow, ap->mf_MainWindow.pConformant);

	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_create_layout_main", 0);
	ap->mf_MainWindow.pMainLayout = mf_widget_create_layout_main(ap->mf_MainWindow.pConformant);
	MF_TA_ACUM_ITEM_END("    __mf_main_create_layout_main", 0);

	elm_object_content_set(ap->mf_MainWindow.pConformant, ap->mf_MainWindow.pMainLayout);
	elm_win_conformant_set(ap->mf_MainWindow.pWindow, EINA_TRUE);
	elm_win_indicator_mode_set(ap->mf_MainWindow.pWindow, ELM_WIN_INDICATOR_SHOW);

	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_storage_status_get", 0);
	__mf_main_storage_status_get(ap);
	MF_TA_ACUM_ITEM_END("    __mf_main_storage_status_get", 0);


	MF_TA_ACUM_ITEM_BEGIN("    create phone naviframe struct", 0);
	mf_widget_phone_storage_init(ap);
	MF_TA_ACUM_ITEM_END("    create phone naviframe struct", 0);

	//create landscape/portrait view
	MF_TA_ACUM_ITEM_BEGIN("    mf_widget_create", 0);
	mf_widget_create(ap);
	MF_TA_ACUM_ITEM_END("    mf_widget_create", 0);

	MF_TA_ACUM_ITEM_BEGIN("    set_view_to_mainlayout", 0);


	elm_object_part_content_set(ap->mf_MainWindow.pMainLayout, "elm.swallow.content", ap->mf_MainWindow.pNaviBar);
#ifdef MYFILE_SPLIT_VIEW
	mf_widget_set_main_layout_content(ap);
#endif
	MF_TA_ACUM_ITEM_END("    set_view_to_mainlayout", 0);


	/*Create content frame of Main Layout*/
	MF_TA_ACUM_ITEM_BEGIN("    media_content_connect", 0);
	ret = media_content_connect();
	mf_retvm_if(ret < 0, -1, "Fail to media_content_connect()");
	MF_TA_ACUM_ITEM_END("    media_content_connect", 0);


	MF_TA_ACUM_ITEM_BEGIN("    mf_callback_create_dir_monitor", 0);
	ret = mf_callback_create_dir_monitor(ap);
	mf_retvm_if(ret < 0, -1, "Fail to mf_callback_create_dir_monitor()");
	MF_TA_ACUM_ITEM_END("    mf_callback_create_dir_monitor", 0);

	MF_TA_ACUM_ITEM_BEGIN("    app_init_idler", 0);
	ap->mf_Status.app_init_idler = ecore_idler_add(__mf_main_app_init_idler_cb, ap);
	MF_TA_ACUM_ITEM_END("    app_init_idler", 0);

	evas_object_smart_callback_add(ap->mf_MainWindow.pWindow, "profile,changed", mf_callback_profile_changed_cb, ap);

	MF_TA_ACUM_ITEM_END("  __mf_main_create_app", 0);
	MF_TA_ACUM_ITEM_BEGIN("  create_to_reset", 0);

	MF_TRACE_END;

	ap->mf_Status.b_run_background = false; /* 20121008 added for fix PLM P121001-0404 temporary */

	return true;
}

static void __mf_main_reset_app(service_h service, void *data)
{
	MF_TRACE_BEGIN;
	MF_TA_ACUM_ITEM_END("  create_to_reset", 0);


	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_reset_app", 0);
	struct appdata *ap = (struct appdata *)data;
	char *operation = NULL;

	MF_TA_ACUM_ITEM_BEGIN("    service_get_operation", 0);

	service_get_operation(service, &operation);
	MF_TA_ACUM_ITEM_END("    service_get_operation", 0);


	if (g_strcmp0(operation, SERVICE_OPERATION_PICK) == 0) {
		__mf_main_bundle_parse(service, ap);
		/*launch myfile ug here*/
		ap->mf_SharedGadget.ug = mf_launch_load_ug_myfile(ap);

		service_clone(&ap->mf_Bundle.recv_service, service);
	} else {
		mf_debug();
		char *launch_type_str = NULL;
		service_get_extra_data(service, "launch-type", &launch_type_str);
		if(launch_type_str &&!strcasecmp(launch_type_str, "shortcut")) {
			SAFE_FREE_CHAR(ap->mf_Bundle.path);
			SAFE_FREE_CHAR(ap->mf_Bundle.select_type);
			SAFE_FREE_CHAR(ap->mf_Bundle.file_type);
			SAFE_FREE_CHAR(ap->mf_Bundle.marked_mode);
			SAFE_FREE_CHAR(ap->mf_Bundle.drm_type);
			ap->mf_Bundle.path = g_strdup(PHONE_FOLDER);
			ap->mf_Bundle.select_type = g_strdup("SHORTCUT");
			ap->mf_SharedGadget.ug = mf_launch_load_ug_myfile(ap);
			SAFE_FREE_CHAR(ap->mf_Bundle.path);
			SAFE_FREE_CHAR(ap->mf_Bundle.select_type);
		} else {
			mf_debug();
			char *uri = NULL;
			service_get_extra_data (service, "path", &uri);
			mf_debug("======== uri is [%s]", uri);
			if (uri && ecore_file_exists(uri)) {
				MF_TA_ACUM_ITEM_BEGIN("    otg_launch", 0);
				myfileNaviBar *pNavi_s_toshow = NULL;
				myfileNaviBar *pNavi_s_inuse = NULL;
				Evas_Object *pNaviBar = NULL;
				ap->mf_Status.view_type = mf_view_normal;
				pNavi_s_inuse = mf_navi_bar_get_in_use(ap);
				if (pNavi_s_inuse == NULL || pNavi_s_inuse->pNaviLabel == NULL) {
					MF_TRACE_END;
					return;
				}

				int locate = mf_fm_svc_wrapper_get_location(uri);
				char *storage = NULL;
				switch (locate) {
				case MYFILE_PHONE:
					storage = GET_SYS_STR(MF_LABEL_PHONE);
					break;
				case MYFILE_MMC:
					storage = GET_SYS_STR(MF_LABEL_MMC);
					break;
				case MYFILE_OTG:
					storage = GET_STR(MF_LABEL_OTG);
				default:
					break;
				}


				pNavi_s_inuse->naviFlagInUse = FALSE;

				pNaviBar = ap->mf_MainWindow.pNaviBar;
				pNavi_s_toshow = mf_navi_bar_get_navi_from_navilist(ap->mf_MainWindow.plistNaviBar, storage);

				if (pNavi_s_toshow == NULL) {
					/*2.0	get root path by storage label */
					char *rootpath = g_strdup(uri);
					if (rootpath == NULL) {
						MF_TRACE_END;
						return;
					}
					/*2.1	create the navi for the tab */
					pNavi_s_toshow = malloc(sizeof(myfileNaviBar));
					if (pNavi_s_toshow == NULL) {
						free(rootpath);
						rootpath = NULL;
						mf_debug("pNavi_s_toshow is NULL");
						MF_TRACE_END;
						return;
					}
					memset(pNavi_s_toshow, 0, sizeof(myfileNaviBar));

					pNavi_s_toshow->naviFlagInUse = TRUE;
					/*2.2	set path as root path of the storage */
					pNavi_s_toshow->pCurrentPath = g_strdup(rootpath);
					pNavi_s_toshow->pNaviLabel = g_strdup(storage);
					free(rootpath);
					/*2.3	insert phone navi into the navi_list */
					ap->mf_MainWindow.plistNaviBar = eina_list_append(ap->mf_MainWindow.plistNaviBar, pNavi_s_toshow);

					if (ap->mf_Status.path != NULL) {
						g_string_free(ap->mf_Status.path, TRUE);
						ap->mf_Status.path = NULL;
					}
					ap->mf_Status.path = g_string_new(pNavi_s_toshow->pCurrentPath);
					__mf_main_remake_app(service, ap);
					MF_TRACE_END;

				} else {
					mf_debug();
					/*2.1	get the navi of the tab */
					pNavi_s_toshow = mf_navi_bar_get_struct_by_label(ap, storage);
					pNavi_s_toshow->naviFlagInUse = TRUE;

					/*2.2	set related status value. */
					if (ap->mf_Status.path != NULL) {
						g_string_free(ap->mf_Status.path, TRUE);
						ap->mf_Status.path = NULL;
					}
					ap->mf_Status.path = g_string_new(uri);
					SAFE_FREE_CHAR(pNavi_s_toshow->pCurrentPath);
					pNavi_s_toshow->pCurrentPath = g_strdup(uri);
					mf_debug();
					/*2.3	update the content to catch update */
					/*Todo: How to ensure insert only once */
					__mf_main_remake_app(service, ap);
					mf_debug();
					mf_navi_bar_remove_previous_contents(ap, ap->mf_MainWindow.pNaviBar);
					MF_TRACE_END;
				}
				MF_TA_ACUM_ITEM_END("	 otg_launch", 0);

			}else {

				/* 20121008 added for fix PLM P121001-0404 temporary { */
				if (ap->mf_MainWindow.pWindow != NULL && ap->mf_Status.b_run_background)
				{
					elm_win_activate(ap->mf_MainWindow.pWindow);
					ap->mf_Status.b_run_background = false;
					return;
				}
				/* 20121008 added for fix PLM P121001-0404 temporary } */
				MF_TA_ACUM_ITEM_BEGIN("    __mf_main_remake_app", 0);
				__mf_main_remake_app(service, ap);
				MF_TA_ACUM_ITEM_END("    __mf_main_remake_app", 0);
			}
		}
	}

	if (ap->mf_MainWindow.pWindow) {
		evas_object_show(ap->mf_MainWindow.pWindow);
		elm_win_activate(ap->mf_MainWindow.pWindow);
	}
	MF_TA_ACUM_ITEM_END("    __mf_main_reset_app", 0);

}

static void __mf_main_resume_app(void *data)
{
	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_resume_app", 0);
	MF_TRACE_BEGIN;
	mf_retm_if(data == NULL, "data is NULL");
#ifdef MYFILE_SPLIT_VIEW
	struct appdata *ap = (struct appdata *)data;
	if (!mf_util_is_rotation_lock()) {
		ap->mf_Status.rotation_type = MF_ROTATE_PORTRAIT;
		ap->mf_Status.rotation_angle = 0;

		mf_callback_app_rotate_cb(APP_DEVICE_ORIENTATION_0, ap);

	} else {
		app_device_orientation_e mode = APP_DEVICE_ORIENTATION_0 ;
		mode = app_get_device_orientation();
		if (mode != ap->mf_Status.rotation_type)
			mf_callback_app_rotate_cb(mode, ap);
	}
	if (ap->mf_MainWindow.pWindow) {
		evas_object_show(ap->mf_MainWindow.pWindow);
	}
#endif
	mf_sbeam_enable();
	MF_TA_ACUM_ITEM_END("    __mf_main_resume_app", 0);
}

/******************************
** Prototype    : __mf_main_terminate_app
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void __mf_main_terminate_app(void *data)
{
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	media_content_disconnect();
	efreet_mime_shutdown();

	if (ap->mf_Status.search_handler)
		mf_search_finalize(&ap->mf_Status.search_handler);

	mf_callback_destory_dir_monitor(ap);

	mf_ecore_idler_del(ap->mf_Status.app_init_idler);
	mf_ecore_idler_del(ap->mf_Status.popup_del_idler);
	mf_ecore_idler_del(ap->mf_Status.navi_content_idler);
	SAFE_DEL_ECORE_TIMER(ap->mf_MainWindow.pPopupTimer);
	SAFE_FREE_ECORE_EVENT(ap->mf_MainWindow.event);
	SAFE_FREE_ECORE_EVENT(ap->mf_MainWindow.font_event);

#ifdef MYFILE_MOTION_FEATURE
	mf_sensor_finalize();
#endif

	MF_TA_ACUM_ITEM_SHOW_RESULT_TO(MF_TA_SHOW_FILE);
	MF_TA_RELEASE();
#ifdef MYFILE_USB_OTG
	mf_otg_finalize(ap);
#endif

#ifdef MYFILE_CRITICAL_LOG
	mf_log_finalize();
#endif
	mf_sbeam_finalize();

	if (ap->mf_FileOperation.job_pop_list != NULL) {	/*job callback must not run before*/
		mf_util_clear_handler_list(ap);
		ap->mf_FileOperation.job_pop_list = NULL;
	}

	if (ap->mf_FileOperation.job_push != NULL) {	/*job callback must not run before*/
		ecore_job_del(ap->mf_FileOperation.job_push);
		ap->mf_FileOperation.job_push = NULL;
	}

	if (ap->mf_Status.flagIcuInit == TRUE)
		mf_util_icu_finalize(ap);

	if (ap->mf_FileOperation.sync_pipe) {
		ecore_pipe_del(ap->mf_FileOperation.sync_pipe);
		ap->mf_FileOperation.sync_pipe = NULL;
	}
	mf_util_set_pm_lock(ap, EINA_FALSE);
	mf_language_mgr_destroy();
}

/******************************
** Prototype    : __mf_main_stop_app
** Description  :
** Input        : void *data
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
void __mf_main_stop_app(void *data)
{
	MF_TA_ACUM_ITEM_BEGIN("    __mf_main_stop_app", 0);
	MF_TRACE_BEGIN;
	struct appdata *ap = (struct appdata *)data;
	assert(ap);

	/*check if phone navi is empty*/
	if (mf_launch_myfile_ug_exist(ap)) {
		/*if myfile ug exists, delete the ug*/
		if (ap->mf_SharedGadget.ug != NULL) {
			ug_destroy(ap->mf_SharedGadget.ug);
			ap->mf_SharedGadget.ug = NULL;
		}

		/*terminate the process*/
		elm_exit();
	} else {
		if (ap->mf_SharedGadget.ug != NULL) {
			ug_destroy(ap->mf_SharedGadget.ug);
			ap->mf_SharedGadget.ug = NULL;
		}
	}

	mf_sbeam_disable();
	MF_TA_ACUM_ITEM_END("    __mf_main_stop_app", 0);

}

/******************************
** Prototype    : main
** Description  :
** Input        : int argc
**                char *argv[]
** Output       : None
** Return Value :
** Calls        :
** Called By    :
**
**  History        :
**  1.Date         : 2010/12/10
**    Author       : Samsung
**    Modification : Created function
**
******************************/
static void __mf_language_changed_cb(void *user_data)
{
	mf_language_mgr_update();
}

int main(int argc, char *argv[])
{
	app_event_callback_s ops;
	struct appdata ad;

	MF_TA_INIT();
	MF_TA_ACUM_ITEM_BEGIN(" Myfiles Launch Time", 0);
	MF_TA_ACUM_ITEM_BEGIN("  main", 0);

#ifdef MYFILE_CRITICAL_LOG
	int ret = mf_log_init();
	if (ret != MYFILE_ERR_NONE)
		mf_debug("initialize critical log failed");
#endif

	memset(&ops, 0x0, sizeof(app_event_callback_s));
	memset(&ad, 0x0, sizeof(struct appdata));

	ops.create = __mf_main_create_app;
	ops.terminate = __mf_main_terminate_app;
	ops.pause = __mf_main_stop_app;
	ops.resume = __mf_main_resume_app;
	ops.service = __mf_main_reset_app;
	ops.device_orientation = mf_callback_app_rotate_cb;
	ops.region_format_changed  = mf_callback_icu_update_cb;
	ops.language_changed = __mf_language_changed_cb;

	MF_TA_ACUM_ITEM_END("  main", 0);
	MF_TA_ACUM_ITEM_BEGIN("  main-to-create", 0);
	return app_efl_main(&argc, &argv, &ops, &ad);
}
