/*
 * Copyright 2013         Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <Elementary.h>

#include "mf-tray-item.h"
#include "mf-util.h"
#include "mf-widget.h"

#ifdef MYFILE_TRAY_FEATURE

#define ICON_NORMAL_SIZE_WIDTH		180.0f
#define ICON_NORMAL_SIZE_HEIGHT		180.0f
#define CATEGORY_IMAGE			"Image"
#define CATEGORY_VIDEO			"Video"
#define CATEGORY_SOUNDS			"Sounds"
#define CATEGORY_DOC			"Document"
#define CATEGORY_OTHERS			"Others"

#define LAYOUT_SWALLOW_BG		"swallow.bg"
#define LAYOUT_INFO_SHORTCUT_ITEM_ICON     "swallow.icon"
static const char *STR_ATTRIBUTE_NAME = "name";
static const char *STR_ATTRIBUTE_ICON_PRESS = "icon_press";
static const char *STR_ATTRIBUTE_ICON = "icon";

static void __mf_tray_item_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	mf_retm_if(obj == NULL, "Invalid argument: parent is NULL\n");
	char *tmp = NULL;

	tmp = evas_object_data_get(obj, STR_ATTRIBUTE_NAME);
	SAFE_FREE_CHAR(tmp);

	tmp = evas_object_data_get(obj, STR_ATTRIBUTE_ICON_PRESS);
	SAFE_FREE_CHAR(tmp);

	mf_debug("_dock_item_del end.\n");
}

int mf_tray_item_type(const char *path)
{
	int type = mf_file_attr_get_file_type_by_mime(path);
	int category = mf_tray_item_category_none;
	switch(type) {
	case FILE_TYPE_IMAGE:
		category = mf_tray_item_category_image;
		break;
	case FILE_TYPE_VIDEO:
		category = mf_tray_item_category_video;
		break;
	case FILE_TYPE_SOUND:
	case FILE_TYPE_MUSIC:
		category = mf_tray_item_category_sounds;
		break;
	case FILE_TYPE_DOC:
	case FILE_TYPE_PDF:
	case FILE_TYPE_PPT:
	case FILE_TYPE_EXCEL:
	case FILE_TYPE_TXT:
		category = mf_tray_item_category_document;
		break;
	case FILE_TYPE_DIR:
		category = mf_tray_item_category_none;
		break;
	default:
		category = mf_tray_item_category_others;
		break;
	}
	mf_debug("path is [%s] category is [%d]", path, category);
	return category;
}

char *mf_tray_item_get_data(Evas_Object *edje, const char *attribute)
{
	return evas_object_data_get(edje, attribute);
}

static void __item_mouse_down_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	mf_retm_if(NULL == data, "NULL == data");
	mf_retm_if(NULL == obj, "NULL == obj");

	Evas_Object *icon = NULL;
	Evas_Object *bg = NULL;
	char *press_icon = NULL;

	bg = elm_object_part_content_get(obj, LAYOUT_SWALLOW_BG);
	icon = elm_object_part_content_get(obj, LAYOUT_INFO_SHORTCUT_ITEM_ICON);
	press_icon = mf_tray_item_get_data(obj, STR_ATTRIBUTE_ICON_PRESS);

	if (bg) {
		mf_debug(" set bg icon %s", MF_ICON_CATEGORY_BG_PRESS);
		elm_image_file_set(bg, MF_ICON_CATEGORY_BG_PRESS, NULL);
	}
	if (icon) {
		elm_image_file_set(icon, press_icon, NULL);
	}

}

static void __item_mouse_up_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	mf_retm_if(NULL == data, "NULL == data");
	mf_retm_if(NULL == obj, "NULL == obj");

	struct appdata *ap = (struct appdata *)data;
	Evas_Object *icon = NULL;
	Evas_Object *bg = NULL;
	char *unpress_icon = NULL;
	const char *name = NULL;

	bg = elm_object_part_content_get(obj, LAYOUT_SWALLOW_BG);
	icon = elm_object_part_content_get(obj, LAYOUT_INFO_SHORTCUT_ITEM_ICON);
	unpress_icon = mf_tray_item_get_data(obj, STR_ATTRIBUTE_ICON);
	name = mf_tray_item_get_data(obj, STR_ATTRIBUTE_NAME);

	if (bg) {
		mf_debug(" set bg icon %s", MF_ICON_CATEGORY_BG);
		elm_image_file_set(bg, MF_ICON_CATEGORY_BG, NULL);
	}
	if (icon) {
		elm_image_file_set(icon, unpress_icon, NULL);
	}

	int category = mf_tray_item_category_none;
	if (g_strcmp0(name, CATEGORY_IMAGE) == 0) {
		category = mf_tray_item_category_image;
	} else if (g_strcmp0(name, CATEGORY_VIDEO) == 0) {
		category = mf_tray_item_category_video;
	} else if (g_strcmp0(name, CATEGORY_SOUNDS) == 0) {
		category = mf_tray_item_category_sounds;
	} else if (g_strcmp0(name, CATEGORY_DOC) == 0) {
		category = mf_tray_item_category_document;
	} else if (g_strcmp0(name, CATEGORY_OTHERS) == 0) {
		category = mf_tray_item_category_others;
	}
	mf_debug("category is [%d]", category);
	if (category != mf_tray_item_category_none) {
		ap->mf_Status.view_type = mf_view_root_category;
		mf_search_bar_category_search(ap, category);
		Evas_Object *btn = mf_widget_unset_part_content(ap->mf_MainWindow.pNaviBar, NAVI_BOTTOM_BUTTON_1_PART);
		SAFE_FREE_OBJ(btn);

		btn = mf_widget_util_create_button(ap->mf_MainWindow.pNaviBar, NAVI_BUTTON_STYLE, MF_LABEL_EDIT, NULL, mf_callback_edit_cb, ap);
		if (btn) {
			elm_object_item_part_content_set(ap->mf_MainWindow.pNaviItem, NAVI_BOTTOM_BUTTON_1_PART, btn);
		}
	}

}

static int __mf_tray_item_register_event(void *data, Evas_Object *obj)
{
	mf_retvm_if(NULL == data, EXIT_FAILURE, "NULL == data");

	evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_DOWN, __item_mouse_down_cb, data);
	//evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_MOVE, _item_mouse_move_cb, app_tray);
	evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_UP, __item_mouse_up_cb, data);
	//evas_object_event_callback_add(obj, EVAS_CALLBACK_FOCUS_OUT, _item_focus_out_cb, app_tray);

	return EXIT_SUCCESS;
}
static void __mf_tray_item_data_set(Evas_Object *obj, const char* attribute, const char *data)
{
	mf_retm_if(NULL == obj, "NULL == obj");
	mf_retm_if(NULL == attribute, "NULL == attribute");
	mf_retm_if(NULL == data, "NULL == data");

	evas_object_data_set(obj, attribute, data);

}

static char *__mf_tray_item_get_press_icon(const char *name)
{
	mf_retvm_if(NULL == name, NULL, "NULL == name");
	if (g_strcmp0(name, CATEGORY_IMAGE) == 0) {
		return MF_ICON_CATEGORY_IMAGE_PRESS;
	} else if (g_strcmp0(name, CATEGORY_VIDEO) == 0) {
		return MF_ICON_CATEGORY_VIDEO_PRESS;
	} else if (g_strcmp0(name, CATEGORY_SOUNDS) == 0) {
		return MF_ICON_CATEGORY_SOUNDS_PRESS;
	} else if (g_strcmp0(name, CATEGORY_DOC) == 0) {
		return MF_ICON_CATEGORY_DOCUMENT_PRESS;
	} else if (g_strcmp0(name, CATEGORY_OTHERS) == 0) {
		return MF_ICON_CATEGORY_OTHERS_PRESS;
	} else {
		return NULL;
	}
}
Evas_Object *mf_tray_item_create(Evas_Object *parent, const char *icon_file, const char *name)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(icon_file == NULL, NULL, "icon_file is NULL");
	mf_retvm_if(name == NULL, NULL, "name is NULL");

	Evas_Object *layout = NULL;
	Evas_Object *bg;

	layout = mf_widget_create_layout(parent, EDJ_NAME, GRP_TRAY_ITEM);

	#if 0
	bg = evas_object_rectangle_add(evas);
	if (!bg) {
		evas_object_del(layout);
		return NULL;
	}

	evas_object_color_set(bg, 0, 0, 0, 0);
	evas_object_resize(bg, ICON_NORMAL_SIZE_WIDTH * elm_config_scale_get(), ICON_NORMAL_SIZE_HEIGHT * elm_config_scale_get());
	evas_object_size_hint_min_set(bg, ICON_NORMAL_SIZE_WIDTH * elm_config_scale_get(), ICON_NORMAL_SIZE_HEIGHT * elm_config_scale_get());
	evas_object_size_hint_max_set(bg, ICON_NORMAL_SIZE_WIDTH * elm_config_scale_get(), ICON_NORMAL_SIZE_HEIGHT * elm_config_scale_get());
	#endif

	bg = elm_image_add(layout);
	if (elm_image_file_set(bg, MF_ICON_CATEGORY_BG, NULL) == EINA_FALSE) {
		mf_error("Icon file is not accessible (%s)\n", MF_ICON_CATEGORY_BG);
		evas_object_del(bg);
		bg = NULL;
	}
	elm_image_resizable_set(bg, EINA_TRUE, EINA_TRUE);
	evas_object_show(bg);
	elm_object_part_content_set(layout, LAYOUT_SWALLOW_BG, bg);

	Evas_Object *icon = NULL;

	icon = elm_image_add(layout);
	if (elm_image_file_set(icon, icon_file, NULL) == EINA_FALSE) {
		mf_error("Icon file is not accessible (%s)\n", icon_file);
		evas_object_del(icon);
		icon = NULL;
	}

	elm_image_resizable_set(icon, EINA_TRUE, EINA_TRUE);
	evas_object_show(icon);

	if (icon) {
		Evas_Object *tmp;
		tmp = mf_widget_unset_part_content(layout, LAYOUT_INFO_SHORTCUT_ITEM_ICON);
		if (tmp) {
			evas_object_del(tmp);
		}

		elm_object_part_content_set(layout, LAYOUT_INFO_SHORTCUT_ITEM_ICON, icon);
	}

	__mf_tray_item_data_set(layout, STR_ATTRIBUTE_NAME, name);
	__mf_tray_item_data_set(layout, STR_ATTRIBUTE_ICON, icon_file);
	__mf_tray_item_data_set(layout, STR_ATTRIBUTE_ICON_PRESS, __mf_tray_item_get_press_icon(name));

	//edje_object_part_text_set(_EDJ(layout), LAYOUT_INFO_SHORTCUT_ITEM_NAME, name);
	evas_object_show(layout);
	//evas_object_event_callback_add(layout, EVAS_CALLBACK_DEL, __mf_tray_item_del, NULL);

	return layout;
}
Evas_Object *mf_tray_create(Evas_Object *parent, void *data)
{
	mf_retvm_if(parent == NULL, NULL, "parent is NULL");
	mf_retvm_if(data == NULL, NULL, "data is NULL");

	Evas_Object *tray_box = elm_box_add(parent);

	elm_box_horizontal_set(tray_box, EINA_TRUE);
	elm_box_align_set(tray_box, 0.0, 0.0);
	evas_object_size_hint_weight_set(tray_box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	Evas_Object *Image = mf_tray_item_create(tray_box, MF_ICON_CATEGORY_IMAGE, CATEGORY_IMAGE);
	elm_box_pack_end(tray_box, Image);
	__mf_tray_item_register_event(data, Image);

	Evas_Object *Video = mf_tray_item_create(tray_box, MF_ICON_CATEGORY_VIDEO, CATEGORY_VIDEO);
	elm_box_pack_end(tray_box, Video);
	__mf_tray_item_register_event(data, Video);

	Evas_Object *Sounds = mf_tray_item_create(tray_box, MF_ICON_CATEGORY_SOUNDS, CATEGORY_SOUNDS);
	elm_box_pack_end(tray_box, Sounds);
	__mf_tray_item_register_event(data, Sounds);

	Evas_Object *Document = mf_tray_item_create(tray_box, MF_ICON_CATEGORY_DOCUMENT, CATEGORY_DOC);
	elm_box_pack_end(tray_box, Document);
	__mf_tray_item_register_event(data, Document);

	Evas_Object *Others = mf_tray_item_create(tray_box, MF_ICON_CATEGORY_OTHERS, CATEGORY_OTHERS);
	elm_box_pack_end(tray_box, Others);
	__mf_tray_item_register_event(data, Others);

	//evas_object_event_callback_add(icon, EVAS_CALLBACK_MOUSE_DOWN, _item_mouse_down_cb, app_tray);
	//evas_object_event_callback_add(icon, EVAS_CALLBACK_MOUSE_MOVE, _item_mouse_move_cb, app_tray);

	//evas_object_event_callback_add(icon, EVAS_CALLBACK_MOUSE_UP, _item_mouse_up_cb, app_tray);
	//evas_object_event_callback_add(icon, EVAS_CALLBACK_FOCUS_OUT, _item_focus_out_cb, app_tray);
	return tray_box;
}
#endif
